@@ -186,8 +186,8 @@ mongo_cursor_next_bsonlist <- function(cursor, n = 1){
 }
 
 #' @useDynLib mongolite R_mongo_cursor_next_page
-mongo_cursor_next_page <- function(cursor, size = 100){
-  .Call(R_mongo_cursor_next_page, cursor, size = size)
+mongo_cursor_next_page <- function(cursor, size = 100, as_json = FALSE){
+  .Call(R_mongo_cursor_next_page, cursor, size = size, as_json = as_json)
 }
 
 #' @useDynLib mongolite R_mongo_collection_find_indexes
