@@ -20,14 +20,14 @@
 #include "b64_ntop.h"
 #include "bson-private.h"
 #include "bson-string.h"
+#include "bson-iso8601-private.h"
 
-#include <stdarg.h>
 #include <string.h>
 #include <math.h>
 
 
 #ifndef BSON_MAX_RECURSION
-#define BSON_MAX_RECURSION 100
+#define BSON_MAX_RECURSION 200
 #endif
 
 
@@ -43,6 +43,13 @@ typedef enum {
 } bson_validate_phase_t;
 
 
+typedef enum {
+   BSON_JSON_MODE_LEGACY,
+   BSON_JSON_MODE_CANONICAL,
+   BSON_JSON_MODE_RELAXED,
+} bson_json_mode_t;
+
+
 /*
  * Structures.
  */
@@ -50,14 +57,17 @@ typedef struct {
    bson_validate_flags_t flags;
    ssize_t err_offset;
    bson_validate_phase_t phase;
+   bson_error_t error;
 } bson_validate_state_t;
 
 
 typedef struct {
    uint32_t count;
    bool keys;
+   ssize_t *err_offset;
    uint32_t depth;
    bson_string_t *str;
+   bson_json_mode_t mode;
 } bson_json_state_t;
 
 
@@ -74,14 +84,16 @@ _bson_as_json_visit_document (const bson_iter_t *iter,
                               const char *key,
                               const bson_t *v_document,
                               void *data);
-
+static char *
+_bson_as_json_visit_all (const bson_t *bson,
+                         size_t *length,
+                         bson_json_mode_t mode);
 
 /*
  * Globals.
  */
 static const uint8_t gZero;
 
-
 /*
  *--------------------------------------------------------------------------
  *
@@ -514,7 +526,7 @@ _bson_append_bson_begin (bson_t *bson,           /* IN */
  *       Complete a call to _bson_append_bson_begin.
  *
  * Returns:
- *       true if successful; otherwise false indicating INT_MAX overflow.
+ *       true if successful.
  *
  * Side effects:
  *       @child is destroyed and no longer valid after calling this
@@ -604,7 +616,7 @@ bson_append_array_begin (bson_t *bson,    /* IN */
  *       function.
  *
  * Returns:
- *       true if successful; otherwise false indicating INT_MAX overflow.
+ *       true if successful.
  *
  * Side effects:
  *       @child is invalid after calling this function.
@@ -1493,6 +1505,41 @@ bson_append_oid (bson_t *bson,
 }
 
 
+/*
+ *--------------------------------------------------------------------------
+ *
+ * _bson_append_regex_options_sorted --
+ *
+ *       Helper to append regex options to a buffer in a sorted order.
+ *       Any duplicate or unsupported options will be ignored.
+ *
+ * Parameters:
+ *       @buffer: Buffer to which sorted options will be appended
+ *       @options: Regex options
+ *
+ * Returns:
+ *       None.
+ *
+ * Side effects:
+ *       None.
+ *
+ *--------------------------------------------------------------------------
+ */
+
+static BSON_INLINE void
+_bson_append_regex_options_sorted (bson_string_t *buffer, /* IN */
+                                   const char *options)   /* IN */
+{
+   const char *c;
+
+   for (c = BSON_REGEX_OPTIONS_SORTED; *c; c++) {
+      if (strchr (options, *c)) {
+         bson_string_append_c (buffer, *c);
+      }
+   }
+}
+
+
 bool
 bson_append_regex (bson_t *bson,
                    const char *key,
@@ -1502,7 +1549,8 @@ bson_append_regex (bson_t *bson,
 {
    static const uint8_t type = BSON_TYPE_REGEX;
    uint32_t regex_len;
-   uint32_t options_len;
+   bson_string_t *options_sorted;
+   bool r;
 
    BSON_ASSERT (bson);
    BSON_ASSERT (key);
@@ -1520,21 +1568,27 @@ bson_append_regex (bson_t *bson,
    }
 
    regex_len = (int) strlen (regex) + 1;
-   options_len = (int) strlen (options) + 1;
+   options_sorted = bson_string_new (NULL);
 
-   return _bson_append (bson,
-                        5,
-                        (1 + key_length + 1 + regex_len + options_len),
-                        1,
-                        &type,
-                        key_length,
-                        key,
-                        1,
-                        &gZero,
-                        regex_len,
-                        regex,
-                        options_len,
-                        options);
+   _bson_append_regex_options_sorted (options_sorted, options);
+
+   r =  _bson_append (bson,
+                      5,
+                      (1 + key_length + 1 + regex_len + options_sorted->len + 1),
+                      1,
+                      &type,
+                      key_length,
+                      key,
+                      1,
+                      &gZero,
+                      regex_len,
+                      regex,
+                      options_sorted->len + 1,
+                      options_sorted->str);
+
+   bson_string_free (options_sorted, true);
+
+   return r;
 }
 
 
@@ -2455,7 +2509,12 @@ _bson_as_json_visit_int32 (const bson_iter_t *iter,
 {
    bson_json_state_t *state = data;
 
-   bson_string_append_printf (state->str, "%" PRId32, v_int32);
+   if (state->mode == BSON_JSON_MODE_CANONICAL) {
+      bson_string_append_printf (
+         state->str, "{ \"$numberInt\" : \"%" PRId32 "\" }", v_int32);
+   } else {
+      bson_string_append_printf (state->str, "%" PRId32, v_int32);
+   }
 
    return false;
 }
@@ -2469,7 +2528,12 @@ _bson_as_json_visit_int64 (const bson_iter_t *iter,
 {
    bson_json_state_t *state = data;
 
-   bson_string_append_printf (state->str, "%" PRId64, v_int64);
+   if (state->mode == BSON_JSON_MODE_CANONICAL) {
+      bson_string_append_printf (
+         state->str, "{ \"$numberLong\" : \"%" PRId64 "\"}", v_int64);
+   } else {
+      bson_string_append_printf (state->str, "%" PRId64, v_int64);
+   }
 
    return false;
 }
@@ -2501,22 +2565,42 @@ _bson_as_json_visit_double (const bson_iter_t *iter,
 {
    bson_json_state_t *state = data;
    bson_string_t *str = state->str;
-   uint32_t start_len = str->len;
+   uint32_t start_len;
+   bool legacy;
 
-#ifdef BSON_NEEDS_SET_OUTPUT_FORMAT
-   unsigned int current_format = _set_output_format (_TWO_DIGIT_EXPONENT);
-#endif
+   /* Determine if legacy (i.e. unwrapped) output should be used. Relaxed mode
+    * will use this for nan and inf values, which we check manually since old
+    * platforms may not have isinf or isnan. */
+   legacy = state->mode == BSON_JSON_MODE_LEGACY ||
+            (state->mode == BSON_JSON_MODE_RELAXED &&
+             !(v_double != v_double || v_double * 0 != 0));
 
-   bson_string_append_printf (str, "%.20g", v_double);
-
-   /* ensure trailing ".0" to distinguish "3" from "3.0" */
-   if (strspn (&str->str[start_len], "0123456789-") == str->len - start_len) {
-      bson_string_append (str, ".0");
+   if (!legacy) {
+      bson_string_append (state->str, "{ \"$numberDouble\" : \"");
    }
 
-#ifdef BSON_NEEDS_SET_OUTPUT_FORMAT
-   _set_output_format (current_format);
-#endif
+   if (!legacy && v_double != v_double) {
+      bson_string_append (str, "NaN");
+   } else if (!legacy && v_double * 0 != 0) {
+      if (v_double > 0) {
+         bson_string_append (str, "Infinity");
+      } else {
+         bson_string_append (str, "-Infinity");
+      }
+   } else {
+      start_len = str->len;
+      bson_string_append_printf (str, "%.15g", v_double);
+
+      /* ensure trailing ".0" to distinguish "3" from "3.0" */
+      if (strspn (&str->str[start_len], "0123456789-") ==
+          str->len - start_len) {
+         bson_string_append (str, ".0");
+      }
+   }
+
+   if (!legacy) {
+      bson_string_append (state->str, "\" }");
+   }
 
    return false;
 }
@@ -2580,11 +2664,21 @@ _bson_as_json_visit_binary (const bson_iter_t *iter,
    b64 = bson_malloc0 (b64_len);
    b64_ntop (v_binary, v_binary_len, b64, b64_len);
 
-   bson_string_append (state->str, "{ \"$binary\" : \"");
-   bson_string_append (state->str, b64);
-   bson_string_append (state->str, "\", \"$type\" : \"");
-   bson_string_append_printf (state->str, "%02x", v_subtype);
-   bson_string_append (state->str, "\" }");
+   if (state->mode == BSON_JSON_MODE_CANONICAL ||
+       state->mode == BSON_JSON_MODE_RELAXED) {
+      bson_string_append (state->str, "{ \"$binary\" : { \"base64\": \"");
+      bson_string_append (state->str, b64);
+      bson_string_append (state->str, "\", \"subType\" : \"");
+      bson_string_append_printf (state->str, "%02x", v_subtype);
+      bson_string_append (state->str, "\" } }");
+   } else {
+      bson_string_append (state->str, "{ \"$binary\" : \"");
+      bson_string_append (state->str, b64);
+      bson_string_append (state->str, "\", \"$type\" : \"");
+      bson_string_append_printf (state->str, "%02x", v_subtype);
+      bson_string_append (state->str, "\" }");
+   }
+
    bson_free (b64);
 
    return false;
@@ -2613,9 +2707,20 @@ _bson_as_json_visit_date_time (const bson_iter_t *iter,
 {
    bson_json_state_t *state = data;
 
-   bson_string_append (state->str, "{ \"$date\" : ");
-   bson_string_append_printf (state->str, "%" PRId64, msec_since_epoch);
-   bson_string_append (state->str, " }");
+   if (state->mode == BSON_JSON_MODE_CANONICAL ||
+       (state->mode == BSON_JSON_MODE_RELAXED && msec_since_epoch < 0)) {
+      bson_string_append (state->str, "{ \"$date\" : { \"$numberLong\" : \"");
+      bson_string_append_printf (state->str, "%" PRId64, msec_since_epoch);
+      bson_string_append (state->str, "\" } }");
+   } else if (state->mode == BSON_JSON_MODE_RELAXED) {
+      bson_string_append (state->str, "{ \"$date\" : \"");
+      _bson_iso8601_date_format (msec_since_epoch, state->str);
+      bson_string_append (state->str, "\" }");
+   } else {
+      bson_string_append (state->str, "{ \"$date\" : ");
+      bson_string_append_printf (state->str, "%" PRId64, msec_since_epoch);
+      bson_string_append (state->str, " }");
+   }
 
    return false;
 }
@@ -2629,12 +2734,30 @@ _bson_as_json_visit_regex (const bson_iter_t *iter,
                            void *data)
 {
    bson_json_state_t *state = data;
+   char *escaped;
 
-   bson_string_append (state->str, "{ \"$regex\" : \"");
-   bson_string_append (state->str, v_regex);
-   bson_string_append (state->str, "\", \"$options\" : \"");
-   bson_string_append (state->str, v_options);
-   bson_string_append (state->str, "\" }");
+   escaped = bson_utf8_escape_for_json (v_regex, -1);
+   if (!escaped) {
+      return true;
+   }
+
+   if (state->mode == BSON_JSON_MODE_CANONICAL ||
+       state->mode == BSON_JSON_MODE_RELAXED) {
+      bson_string_append (state->str,
+                          "{ \"$regularExpression\" : { \"pattern\" : \"");
+      bson_string_append (state->str, escaped);
+      bson_string_append (state->str, "\", \"options\" : \"");
+      _bson_append_regex_options_sorted (state->str, v_options);
+      bson_string_append (state->str, "\" } }");
+   } else {
+      bson_string_append (state->str, "{ \"$regex\" : \"");
+      bson_string_append (state->str, escaped);
+      bson_string_append (state->str, "\", \"$options\" : \"");
+      _bson_append_regex_options_sorted (state->str, v_options);
+      bson_string_append (state->str, "\" }");
+   }
+
+   bson_free (escaped);
 
    return false;
 }
@@ -2668,20 +2791,44 @@ _bson_as_json_visit_dbpointer (const bson_iter_t *iter,
                                void *data)
 {
    bson_json_state_t *state = data;
+   char *escaped;
    char str[25];
 
-   bson_string_append (state->str, "{ \"$ref\" : \"");
-   bson_string_append (state->str, v_collection);
-   bson_string_append (state->str, "\"");
-
-   if (v_oid) {
-      bson_oid_to_string (v_oid, str);
-      bson_string_append (state->str, ", \"$id\" : \"");
-      bson_string_append (state->str, str);
-      bson_string_append (state->str, "\"");
+   escaped = bson_utf8_escape_for_json (v_collection, -1);
+   if (!escaped) {
+      return true;
    }
 
-   bson_string_append (state->str, " }");
+   if (state->mode == BSON_JSON_MODE_CANONICAL ||
+       state->mode == BSON_JSON_MODE_RELAXED) {
+      bson_string_append (state->str, "{ \"$dbPointer\" : { \"$ref\" : \"");
+      bson_string_append (state->str, escaped);
+      bson_string_append (state->str, "\"");
+
+      if (v_oid) {
+         bson_oid_to_string (v_oid, str);
+         bson_string_append (state->str, ", \"$id\" : { \"$oid\" : \"");
+         bson_string_append (state->str, str);
+         bson_string_append (state->str, "\" }");
+      }
+
+      bson_string_append (state->str, " } }");
+   } else {
+      bson_string_append (state->str, "{ \"$ref\" : \"");
+      bson_string_append (state->str, escaped);
+      bson_string_append (state->str, "\"");
+
+      if (v_oid) {
+         bson_oid_to_string (v_oid, str);
+         bson_string_append (state->str, ", \"$id\" : \"");
+         bson_string_append (state->str, str);
+         bson_string_append (state->str, "\"");
+      }
+
+      bson_string_append (state->str, " }");
+   }
+
+   bson_free (escaped);
 
    return false;
 }
@@ -2743,6 +2890,13 @@ _bson_as_json_visit_before (const bson_iter_t *iter,
 }
 
 
+static void
+_bson_as_json_visit_corrupt (const bson_iter_t *iter, void *data)
+{
+   *(((bson_json_state_t *) data)->err_offset) = iter->off;
+}
+
+
 static bool
 _bson_as_json_visit_code (const bson_iter_t *iter,
                           const char *key,
@@ -2754,31 +2908,46 @@ _bson_as_json_visit_code (const bson_iter_t *iter,
    char *escaped;
 
    escaped = bson_utf8_escape_for_json (v_code, v_code_len);
-
-   if (escaped) {
-      bson_string_append (state->str, "{ \"$code\" : \"");
-      bson_string_append (state->str, escaped);
-      bson_string_append (state->str, "\" }");
-      bson_free (escaped);
-      return false;
+   if (!escaped) {
+      return true;
    }
 
-   return true;
+   bson_string_append (state->str, "{ \"$code\" : \"");
+   bson_string_append (state->str, escaped);
+   bson_string_append (state->str, "\" }");
+   bson_free (escaped);
+
+   return false;
 }
 
 
 static bool
 _bson_as_json_visit_symbol (const bson_iter_t *iter,
-                            const char *key,
-                            size_t v_symbol_len,
-                            const char *v_symbol,
-                            void *data)
+                                     const char *key,
+                                     size_t v_symbol_len,
+                                     const char *v_symbol,
+                                     void *data)
 {
    bson_json_state_t *state = data;
+   char *escaped;
 
-   bson_string_append (state->str, "\"");
-   bson_string_append (state->str, v_symbol);
-   bson_string_append (state->str, "\"");
+   escaped = bson_utf8_escape_for_json (v_symbol, v_symbol_len);
+   if (!escaped) {
+      return true;
+   }
+
+   if (state->mode == BSON_JSON_MODE_CANONICAL ||
+       state->mode == BSON_JSON_MODE_RELAXED) {
+      bson_string_append (state->str, "{ \"$symbol\" : \"");
+      bson_string_append (state->str, escaped);
+      bson_string_append (state->str, "\" }");
+   } else {
+      bson_string_append (state->str, "\"");
+      bson_string_append (state->str, escaped);
+      bson_string_append (state->str, "\"");
+   }
+
+   bson_free (escaped);
 
    return false;
 }
@@ -2798,10 +2967,12 @@ _bson_as_json_visit_codewscope (const bson_iter_t *iter,
 
    code_escaped = bson_utf8_escape_for_json (v_code, v_code_len);
    if (!code_escaped) {
-      return true; /* error */
+      return true;
    }
 
-   scope = bson_as_json (v_scope, NULL);
+   /* Encode scope with the same mode */
+   scope = _bson_as_json_visit_all (v_scope, NULL, state->mode);
+
    if (!scope) {
       bson_free (code_escaped);
       return true;
@@ -2823,7 +2994,7 @@ _bson_as_json_visit_codewscope (const bson_iter_t *iter,
 static const bson_visitor_t bson_as_json_visitors = {
    _bson_as_json_visit_before,
    NULL, /* visit_after */
-   NULL, /* visit_corrupt */
+   _bson_as_json_visit_corrupt,
    _bson_as_json_visit_double,
    _bson_as_json_visit_utf8,
    _bson_as_json_visit_document,
@@ -2856,7 +3027,7 @@ _bson_as_json_visit_document (const bson_iter_t *iter,
                               void *data)
 {
    bson_json_state_t *state = data;
-   bson_json_state_t child_state = {0, true};
+   bson_json_state_t child_state = {0, true, state->err_offset};
    bson_iter_t child;
 
    if (state->depth >= BSON_MAX_RECURSION) {
@@ -2867,7 +3038,11 @@ _bson_as_json_visit_document (const bson_iter_t *iter,
    if (bson_iter_init (&child, v_document)) {
       child_state.str = bson_string_new ("{ ");
       child_state.depth = state->depth + 1;
-      bson_iter_visit_all (&child, &bson_as_json_visitors, &child_state);
+      child_state.mode = state->mode;
+      if (bson_iter_visit_all (&child, &bson_as_json_visitors, &child_state)) {
+         return true;
+      }
+
       bson_string_append (child_state.str, " }");
       bson_string_append (state->str, child_state.str->str);
       bson_string_free (child_state.str, true);
@@ -2884,7 +3059,7 @@ _bson_as_json_visit_array (const bson_iter_t *iter,
                            void *data)
 {
    bson_json_state_t *state = data;
-   bson_json_state_t child_state = {0, false};
+   bson_json_state_t child_state = {0, false, state->err_offset};
    bson_iter_t child;
 
    if (state->depth >= BSON_MAX_RECURSION) {
@@ -2895,7 +3070,11 @@ _bson_as_json_visit_array (const bson_iter_t *iter,
    if (bson_iter_init (&child, v_array)) {
       child_state.str = bson_string_new ("[ ");
       child_state.depth = state->depth + 1;
-      bson_iter_visit_all (&child, &bson_as_json_visitors, &child_state);
+      child_state.mode = state->mode;
+      if (bson_iter_visit_all (&child, &bson_as_json_visitors, &child_state)) {
+         return true;
+      }
+
       bson_string_append (child_state.str, " ]");
       bson_string_append (state->str, child_state.str->str);
       bson_string_free (child_state.str, true);
@@ -2905,11 +3084,14 @@ _bson_as_json_visit_array (const bson_iter_t *iter,
 }
 
 
-char *
-bson_as_json (const bson_t *bson, size_t *length)
+static char *
+_bson_as_json_visit_all (const bson_t *bson,
+                         size_t *length,
+                         bson_json_mode_t mode)
 {
    bson_json_state_t state;
    bson_iter_t iter;
+   ssize_t err_offset = -1;
 
    BSON_ASSERT (bson);
 
@@ -2933,9 +3115,11 @@ bson_as_json (const bson_t *bson, size_t *length)
    state.keys = true;
    state.str = bson_string_new ("{ ");
    state.depth = 0;
+   state.err_offset = &err_offset;
+   state.mode = mode;
 
    if (bson_iter_visit_all (&iter, &bson_as_json_visitors, &state) ||
-       iter.err_off) {
+       err_offset != -1) {
       /*
        * We were prematurely exited due to corruption or failed visitor.
        */
@@ -2956,11 +3140,33 @@ bson_as_json (const bson_t *bson, size_t *length)
 }
 
 
+char *
+bson_as_canonical_extended_json (const bson_t *bson, size_t *length)
+{
+   return _bson_as_json_visit_all (bson, length, BSON_JSON_MODE_CANONICAL);
+}
+
+
+char *
+bson_as_json (const bson_t *bson, size_t *length)
+{
+   return _bson_as_json_visit_all (bson, length, BSON_JSON_MODE_LEGACY);
+}
+
+
+char *
+bson_as_relaxed_extended_json (const bson_t *bson, size_t *length)
+{
+   return _bson_as_json_visit_all (bson, length, BSON_JSON_MODE_RELAXED);
+}
+
+
 char *
 bson_array_as_json (const bson_t *bson, size_t *length)
 {
    bson_json_state_t state;
    bson_iter_t iter;
+   ssize_t err_offset = -1;
 
    BSON_ASSERT (bson);
 
@@ -2984,10 +3190,12 @@ bson_array_as_json (const bson_t *bson, size_t *length)
    state.keys = false;
    state.str = bson_string_new ("[ ");
    state.depth = 0;
+   state.err_offset = &err_offset;
+   state.mode = BSON_JSON_MODE_LEGACY;
    bson_iter_visit_all (&iter, &bson_as_json_visitors, &state);
 
    if (bson_iter_visit_all (&iter, &bson_as_json_visitors, &state) ||
-       iter.err_off) {
+       err_offset != -1) {
       /*
        * We were prematurely exited due to corruption or failed visitor.
        */
@@ -3008,6 +3216,9 @@ bson_array_as_json (const bson_t *bson, size_t *length)
 }
 
 
+#define VALIDATION_ERR(_flag, _msg, ...) \
+   bson_set_error (&state->error, BSON_ERROR_INVALID, _flag, _msg, __VA_ARGS__)
+
 static bool
 _bson_iter_validate_utf8 (const bson_iter_t *iter,
                           const char *key,
@@ -3023,6 +3234,8 @@ _bson_iter_validate_utf8 (const bson_iter_t *iter,
 
       if (!bson_utf8_validate (v_utf8, v_utf8_len, allow_null)) {
          state->err_offset = iter->off;
+         VALIDATION_ERR (
+            BSON_VALIDATE_UTF8, "invalid utf8 string for key \"%s\"", key);
          return true;
       }
    }
@@ -3045,6 +3258,7 @@ _bson_iter_validate_corrupt (const bson_iter_t *iter, void *data)
    bson_validate_state_t *state = data;
 
    state->err_offset = iter->err_off;
+   VALIDATION_ERR (BSON_VALIDATE_NONE, "%s", "corrupt BSON");
 }
 
 
@@ -3058,6 +3272,7 @@ _bson_iter_validate_before (const bson_iter_t *iter,
    if ((state->flags & BSON_VALIDATE_EMPTY_KEYS)) {
       if (key[0] == '\0') {
          state->err_offset = iter->off;
+         VALIDATION_ERR (BSON_VALIDATE_EMPTY_KEYS, "%s", "empty key");
          return true;
       }
    }
@@ -3075,12 +3290,18 @@ _bson_iter_validate_before (const bson_iter_t *iter,
             state->phase = BSON_VALIDATE_PHASE_LF_DB_UTF8;
          } else {
             state->err_offset = iter->off;
+            VALIDATION_ERR (BSON_VALIDATE_DOLLAR_KEYS,
+                            "keys cannot begin with \"$\": \"%s\"",
+                            key);
             return true;
          }
       } else if (state->phase == BSON_VALIDATE_PHASE_LF_ID_KEY ||
                  state->phase == BSON_VALIDATE_PHASE_LF_REF_UTF8 ||
                  state->phase == BSON_VALIDATE_PHASE_LF_DB_UTF8) {
          state->err_offset = iter->off;
+         VALIDATION_ERR (BSON_VALIDATE_DOLLAR_KEYS,
+                         "invalid key within DBRef subdocument: \"%s\"",
+                         key);
          return true;
       } else {
          state->phase = BSON_VALIDATE_PHASE_NOT_DBREF;
@@ -3090,6 +3311,8 @@ _bson_iter_validate_before (const bson_iter_t *iter,
    if ((state->flags & BSON_VALIDATE_DOT_KEYS)) {
       if (strstr (key, ".")) {
          state->err_offset = iter->off;
+         VALIDATION_ERR (
+            BSON_VALIDATE_DOT_KEYS, "keys cannot contain \".\": \"%s\"", key);
          return true;
       }
    }
@@ -3107,10 +3330,11 @@ _bson_iter_validate_codewscope (const bson_iter_t *iter,
                                 void *data)
 {
    bson_validate_state_t *state = data;
-   size_t offset;
+   size_t offset = 0;
 
    if (!bson_validate (v_scope, state->flags, &offset)) {
       state->err_offset = iter->off + offset;
+      VALIDATION_ERR (BSON_VALIDATE_NONE, "%s", "corrupt code-with-scope");
       return false;
    }
 
@@ -3173,7 +3397,10 @@ _bson_iter_validate_document (const bson_iter_t *iter,
    if (state->phase == BSON_VALIDATE_PHASE_LF_ID_KEY ||
        state->phase == BSON_VALIDATE_PHASE_LF_REF_UTF8 ||
        state->phase == BSON_VALIDATE_PHASE_LF_DB_UTF8) {
-      state->err_offset = iter->off;
+      if (state->err_offset <= 0) {
+         state->err_offset = iter->off;
+      }
+
       return true;
    }
 
@@ -3183,23 +3410,52 @@ _bson_iter_validate_document (const bson_iter_t *iter,
 }
 
 
+static void
+_bson_validate_internal (const bson_t *bson, bson_validate_state_t *state)
+{
+   bson_iter_t iter;
+
+   state->err_offset = -1;
+   state->phase = BSON_VALIDATE_PHASE_START;
+   memset (&state->error, 0, sizeof state->error);
+
+   if (!bson_iter_init (&iter, bson)) {
+      state->err_offset = 0;
+      VALIDATION_ERR (BSON_VALIDATE_NONE, "%s", "corrupt BSON");
+   } else {
+      _bson_iter_validate_document (&iter, NULL, bson, state);
+   }
+}
+
+
 bool
 bson_validate (const bson_t *bson, bson_validate_flags_t flags, size_t *offset)
 {
-   bson_validate_state_t state = {flags, -1, BSON_VALIDATE_PHASE_START};
-   bson_iter_t iter;
+   bson_validate_state_t state;
 
-   if (!bson_iter_init (&iter, bson)) {
-      state.err_offset = 0;
-      goto failure;
+   state.flags = flags;
+   _bson_validate_internal (bson, &state);
+
+   if (state.err_offset > 0 && offset) {
+      *offset = (size_t) state.err_offset;
    }
 
-   _bson_iter_validate_document (&iter, NULL, bson, &state);
+   return state.err_offset < 0;
+}
 
-failure:
 
-   if (offset) {
-      *offset = state.err_offset;
+bool
+bson_validate_with_error (const bson_t *bson,
+                          bson_validate_flags_t flags,
+                          bson_error_t *error)
+{
+   bson_validate_state_t state;
+
+   state.flags = flags;
+   _bson_validate_internal (bson, &state);
+
+   if (state.err_offset > 0 && error) {
+      memcpy (error, &state.error, sizeof *error);
    }
 
    return state.err_offset < 0;
