@@ -21,6 +21,7 @@
 
 #include <bson.h>
 
+#include "mongoc-macros.h"
 #include "mongoc-write-concern.h"
 
 #define MONGOC_BULK_WRITE_FLAGS_INIT                     \
@@ -30,78 +31,85 @@
 
 BSON_BEGIN_DECLS
 
+/* forward decl */
+struct _mongoc_client_session_t;
 
 typedef struct _mongoc_bulk_operation_t mongoc_bulk_operation_t;
 typedef struct _mongoc_bulk_write_flags_t mongoc_bulk_write_flags_t;
 
 
-BSON_EXPORT (void)
+MONGOC_EXPORT (void)
 mongoc_bulk_operation_destroy (mongoc_bulk_operation_t *bulk);
-BSON_EXPORT (uint32_t)
+MONGOC_EXPORT (uint32_t)
 mongoc_bulk_operation_execute (mongoc_bulk_operation_t *bulk,
                                bson_t *reply,
                                bson_error_t *error);
-BSON_EXPORT (void)
+MONGOC_EXPORT (void)
 mongoc_bulk_operation_delete (mongoc_bulk_operation_t *bulk,
                               const bson_t *selector)
    BSON_GNUC_DEPRECATED_FOR (mongoc_bulk_operation_remove);
-BSON_EXPORT (void)
+MONGOC_EXPORT (void)
 mongoc_bulk_operation_delete_one (mongoc_bulk_operation_t *bulk,
                                   const bson_t *selector)
    BSON_GNUC_DEPRECATED_FOR (mongoc_bulk_operation_remove_one);
-BSON_EXPORT (void)
+MONGOC_EXPORT (void)
 mongoc_bulk_operation_insert (mongoc_bulk_operation_t *bulk,
                               const bson_t *document);
-BSON_EXPORT (void)
+MONGOC_EXPORT (bool)
+mongoc_bulk_operation_insert_with_opts (mongoc_bulk_operation_t *bulk,
+                                        const bson_t *document,
+                                        const bson_t *opts,
+                                        bson_error_t *error); /* OUT */
+MONGOC_EXPORT (void)
 mongoc_bulk_operation_remove (mongoc_bulk_operation_t *bulk,
                               const bson_t *selector);
-BSON_EXPORT (bool)
+MONGOC_EXPORT (bool)
 mongoc_bulk_operation_remove_many_with_opts (mongoc_bulk_operation_t *bulk,
                                              const bson_t *selector,
                                              const bson_t *opts,
                                              bson_error_t *error); /* OUT */
-BSON_EXPORT (void)
+MONGOC_EXPORT (void)
 mongoc_bulk_operation_remove_one (mongoc_bulk_operation_t *bulk,
                                   const bson_t *selector);
-BSON_EXPORT (bool)
+MONGOC_EXPORT (bool)
 mongoc_bulk_operation_remove_one_with_opts (mongoc_bulk_operation_t *bulk,
                                             const bson_t *selector,
                                             const bson_t *opts,
                                             bson_error_t *error); /* OUT */
-BSON_EXPORT (void)
+MONGOC_EXPORT (void)
 mongoc_bulk_operation_replace_one (mongoc_bulk_operation_t *bulk,
                                    const bson_t *selector,
                                    const bson_t *document,
                                    bool upsert);
-BSON_EXPORT (bool)
+MONGOC_EXPORT (bool)
 mongoc_bulk_operation_replace_one_with_opts (mongoc_bulk_operation_t *bulk,
                                              const bson_t *selector,
                                              const bson_t *document,
                                              const bson_t *opts,
                                              bson_error_t *error); /* OUT */
-BSON_EXPORT (void)
+MONGOC_EXPORT (void)
 mongoc_bulk_operation_update (mongoc_bulk_operation_t *bulk,
                               const bson_t *selector,
                               const bson_t *document,
                               bool upsert);
-BSON_EXPORT (bool)
+MONGOC_EXPORT (bool)
 mongoc_bulk_operation_update_many_with_opts (mongoc_bulk_operation_t *bulk,
                                              const bson_t *selector,
                                              const bson_t *document,
                                              const bson_t *opts,
                                              bson_error_t *error); /* OUT */
-BSON_EXPORT (void)
+MONGOC_EXPORT (void)
 mongoc_bulk_operation_update_one (mongoc_bulk_operation_t *bulk,
                                   const bson_t *selector,
                                   const bson_t *document,
                                   bool upsert);
-BSON_EXPORT (bool)
+MONGOC_EXPORT (bool)
 mongoc_bulk_operation_update_one_with_opts (mongoc_bulk_operation_t *bulk,
                                             const bson_t *selector,
                                             const bson_t *document,
                                             const bson_t *opts,
                                             bson_error_t *error); /* OUT */
-BSON_EXPORT (void)
+MONGOC_EXPORT (void)
 mongoc_bulk_operation_set_bypass_document_validation (
    mongoc_bulk_operation_t *bulk, bool bypass);
 
@@ -111,27 +119,31 @@ mongoc_bulk_operation_set_bypass_document_validation (
  * those wanting to replay a bulk operation to a number of clients or
  * collections.
  */
-BSON_EXPORT (mongoc_bulk_operation_t *)
+MONGOC_EXPORT (mongoc_bulk_operation_t *)
 mongoc_bulk_operation_new (bool ordered);
-BSON_EXPORT (void)
+MONGOC_EXPORT (void)
 mongoc_bulk_operation_set_write_concern (
    mongoc_bulk_operation_t *bulk, const mongoc_write_concern_t *write_concern);
-BSON_EXPORT (void)
+MONGOC_EXPORT (void)
 mongoc_bulk_operation_set_database (mongoc_bulk_operation_t *bulk,
                                     const char *database);
-BSON_EXPORT (void)
+MONGOC_EXPORT (void)
 mongoc_bulk_operation_set_collection (mongoc_bulk_operation_t *bulk,
                                       const char *collection);
-BSON_EXPORT (void)
+MONGOC_EXPORT (void)
 mongoc_bulk_operation_set_client (mongoc_bulk_operation_t *bulk, void *client);
+MONGOC_EXPORT (void)
+mongoc_bulk_operation_set_client_session (
+   mongoc_bulk_operation_t *bulk,
+   struct _mongoc_client_session_t *client_session);
 /* These names include the term "hint" for backward compatibility, should be
  * mongoc_bulk_operation_get_server_id, mongoc_bulk_operation_set_server_id. */
-BSON_EXPORT (void)
+MONGOC_EXPORT (void)
 mongoc_bulk_operation_set_hint (mongoc_bulk_operation_t *bulk,
                                 uint32_t server_id);
-BSON_EXPORT (uint32_t)
+MONGOC_EXPORT (uint32_t)
 mongoc_bulk_operation_get_hint (const mongoc_bulk_operation_t *bulk);
-BSON_EXPORT (const mongoc_write_concern_t *)
+MONGOC_EXPORT (const mongoc_write_concern_t *)
 mongoc_bulk_operation_get_write_concern (const mongoc_bulk_operation_t *bulk);
 BSON_END_DECLS
 
