@@ -23,10 +23,8 @@
 
 #include <bson.h>
 
-#include "mongoc-buffer-private.h"
 #include "mongoc-client.h"
 
-
 BSON_BEGIN_DECLS
 
 
@@ -37,7 +35,6 @@ struct _mongoc_collection_t {
    char db[128];
    char collection[128];
    uint32_t collectionlen;
-   mongoc_buffer_t buffer;
    mongoc_read_prefs_t *read_prefs;
    mongoc_read_concern_t *read_concern;
    mongoc_write_concern_t *write_concern;
@@ -53,8 +50,7 @@ _mongoc_collection_new (mongoc_client_t *client,
                         const mongoc_read_concern_t *read_concern,
                         const mongoc_write_concern_t *write_concern);
 mongoc_cursor_t *
-_mongoc_collection_find_indexes_legacy (mongoc_collection_t *collection,
-                                        bson_error_t *error);
+_mongoc_collection_find_indexes_legacy (mongoc_collection_t *collection);
 
 
 BSON_END_DECLS
